"""
Code to plot the probability distribution

To run:
    From inside the directory 'PyGBeOrientation_1PGBsampling' run:
        
    'python plot_probability.py folder_where_data_is'

For the results in the paper, the data is in the folders:
'1IGTsigma_*'

'plot_probability.py' supports data that is in multiple files, 
in case you are making runs in different machines.

The code 'run_angle.py' stores the data in 'outputs' by default.
Then, you would run: 'python plot_probability.py outputs'

If you want the labels for kappa and sigma to be correct, please
follow the naming convention used in the folders containing the
data from the paper (for example: 1IGTsigma_-0.05_kappa0.125)

"""
import sys
import os
from numpy import *
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt

def rotation_integration(U, tilt, rot):

    i=0
    da_rot = 0
    while da_rot==0:
        da_rot = rot[i+1]-rot[i]
        i+=1
    print 'da_rot = %f'%(da_rot)

    nrot = int(360/da_rot)
    point_prob = []
    point_tilt = []
    point_rot  = []

    kT = 2.479/4.184
    rot_integral = []
    angles = []
    da_tilt = []
    for i in range(len(tilt)):
        # Extreme cases: tilt = 0 and tilt = 180
        if abs(tilt[i])<1e-10:
            rot_integral.append(exp(-U[i]/kT)*2*pi)
            angles.append(tilt[i])
            if i+1==len(tilt):
                da_tilt.append((tilt[i]-tilt[i-1])/2.)
            else:
                da_tilt.append((tilt[i+1]-tilt[i])/2.)
            point_prob.extend(ones(nrot)*exp(-U[i]/kT)*da_tilt[-1]*da_rot*(pi/180)**2)
            point_tilt.extend(ones(nrot)*tilt[i])
            point_rot.extend(ones(nrot)*arange(0, 360, da_rot))

        elif abs(tilt[i]-180)<1e-10:
            rot_integral.append(exp(-U[i]/kT)*2*pi)
            angles.append(tilt[i])
            if i+1==len(tilt):
                da_tilt.append((tilt[i]-tilt[i-1])/2.)
            else:
                da_tilt.append((tilt[i+1]-tilt[i])/2.)
            point_prob.extend(ones(nrot)*exp(-U[i]/kT)*da_tilt[-1]*da_rot*(pi/180)**2)
            point_tilt.extend(ones(nrot)*tilt[i])
            point_rot.extend(ones(nrot)*arange(0, 360, da_rot))
            
        elif abs(tilt[i]-tilt[i-1])<1e-10 and i!=0:
            rot_integral[-1] += exp(-U[i]/kT)*da_rot*pi/180.
            point_prob.append(exp(-U[i]/kT)*da_tilt[-1]*da_rot*(pi/180)**2)
            point_tilt.append(tilt[i])
            point_rot.append(rot[i])
        else:
            rot_integral.append(exp(-U[i]/kT)*da_rot*pi/180.)
            angles.append(tilt[i])
            if i==0:
                ii=0
                da_tilt_aux = 0
                while da_tilt_aux==0:
                    da_tilt_aux = tilt[ii+1]-tilt[ii]
                    ii+=1
                da_tilt.append(da_tilt_aux)
            else:
                da_tilt.append(tilt[i]-tilt[i-1])
            point_prob.append(rot_integral[-1]*da_tilt[-1]*pi/180)
            point_tilt.append(tilt[i])
            point_rot.append(rot[i])

    print '%i tilt angles, from %f to %f in steps of %f'%(len(angles), angles[0], angles[-1], angles[1]-angles[0])

    point_prob = array(point_prob)
    point_tilt = array(point_tilt)
    point_rot = array(point_rot)
    return rot_integral, da_tilt, angles, point_prob, point_tilt, point_rot

def tilt_integration(rot_integral, angles, angle_lim):

    tilt_integral = zeros(len(angle_lim)-1)
    for i in range(len(tilt_integral)):
        low = angle_lim[i]
        upp = angle_lim[i+1]
        #print '\nlow: %f, upp: %f'%(low,upp)
        low_limit=False

        for j in range(len(angles)-1):
            da_tilt = angles[j+1] - angles[j]
            if low<=angles[0]: 
                low = angles[0]+1e-10 
            if low>angles[j] and low <= angles[j+1]:
                #print 'Lower lim between %f and %f'%(angles[j],angles[j+1])
                if upp<angles[j+1]:
                    factor = (upp-low)/(angles[j+1]-angles[j])
                    tilt_integral[i] += factor*da_tilt*pi/180.*(rot_integral[j]+rot_integral[j+1])/2
                    #print 'Upper lim also between %f and %f'%(angles[j],angles[j+1])
                    break
                else:
                    factor = (angles[j+1]-low)/(angles[j+1]-angles[j])
                    tilt_integral[i] += factor*da_tilt*pi/180.*(rot_integral[j]+rot_integral[j+1])/2
                    low_limit = True


            elif low_limit==True and upp>angles[j+1]:
                tilt_integral[i] += da_tilt*pi/180.*(rot_integral[j]+rot_integral[j+1])/2
                #print 'Full integration between %f and %f'%(angles[j],angles[j+1])

            elif upp <= angles[j+1]:
                factor = (upp-angles[j])/(angles[j+1]-angles[j])
                tilt_integral[i] += factor*da_tilt*pi/180.*(rot_integral[j]+rot_integral[j+1])/2
                #print 'Upper between %f and %f'%(angles[j],angles[j+1])
                break

    return tilt_integral


folder = sys.argv[1]

if folder[-1] == '/':
    folder = folder[:-1]

comm = 'python processData.py '+folder
os.system(comm)

file_pos = folder+'_data'

if file_pos[10]=='-':
    try:
        sigma_pos = float(file_pos[10:15])
    except ValueError:
        sigma_pos = 9999
    try:
        kappa = float(file_pos[21:-5])
    except ValueError:
        kappa = 9999
else:
    try:
        sigma_pos = float(file_pos[10:14])
    except ValueError:
        sigma_pos = 9999
    try:
        kappa = float(file_pos[20:-5])
    except ValueError:
        kappa = 9999

print 'sigma: %f'%sigma_pos
print 'kappa: %f'%kappa
    

result_pos = loadtxt(file_pos)
os.system('rm '+file_pos)


tilt_p = result_pos[:,0]
rot_p  = result_pos[:,1]
cal_p  = result_pos[:,2]


av_cal_p = average(cal_p)
cal_p -= av_cal_p
#cal_p -= min(cal_p)

print 'Positive pass'
rot_integral_p, da_tilt_p, angles_p, prob_point_p, til_point_p, rot_point_p = rotation_integration(cal_p, tilt_p, rot_p)
print 'Integration goes from %f to %f'%(tilt_p[0], tilt_p[-1])

# Integration limits
h = 0.05
cos_th = arange(-1, 1+h, h)
cos_th[-1] = 1.
cos_th[0] = -1.


cos_th = flipud(cos_th) # Flip so that angles are in increasing order
angle_lim_cos = arccos(cos_th)*180/pi
dangle = 4
angle_lim_alpha = arange(0,180+dangle/2,dangle)

tilt_integral_p_cos = tilt_integration(rot_integral_p, angles_p, angle_lim_cos) 
tilt_integral_p_alpha = tilt_integration(rot_integral_p, angles_p, angle_lim_alpha) 

cos_th_plot = zeros(len(cos_th)-1)
for i in range(len(cos_th_plot)):
    cos_th_plot[i] = (cos_th[i] + cos_th[i+1])/2

alpha_plot = zeros(len(angle_lim_alpha)-1)
for i in range(len(alpha_plot)):
    alpha_plot[i] = (angle_lim_alpha[i] + angle_lim_alpha[i+1])/2

Zp_cos = sum(tilt_integral_p_cos)
prob_p_cos = tilt_integral_p_cos/Zp_cos

Zp_alpha = sum(tilt_integral_p_alpha)
prob_p_alpha = tilt_integral_p_alpha/Zp_alpha

cos_th_avg_p = sum(cos_th_plot*prob_p_cos)
alpha_avg_p = sum(alpha_plot*prob_p_alpha)

print 'cos(alpha):'
print 'Average cos(th): %f'%(cos_th_avg_p)
print 'Max probability: %f'%(max(prob_p_cos))

print 'alpha:'
print 'Average angle  : %f'%(alpha_avg_p)
print 'Max probability: %f'%(max(prob_p_alpha))

font = {'family':'serif','size':10}

plt.rc('font',**font)

# Plot cos of angle
fig = plt.figure(figsize=(3,2), dpi=80)
ax = fig.add_subplot(111)
ax.plot(cos_th_plot, prob_p_cos, c='k', marker='', lw=0.8, ls=':', label=r'$\sigma$=%1.2f C/m$^2$'%sigma_pos)
for axis in ['top','bottom','left','right']:
    ax.spines[axis].set_linewidth(0.5)
ax.set_ylabel('Probability')
ax.set_xlabel(r'cos($\alpha_{\rm tilt}$)')
ax.legend(fontsize=7, handlelength=2, loc='upper center').get_frame().set_lw(0.3)
#ax.text(0, max(prob_n_cos)/2,r'$\langle$cos($\alpha_{\rm tilt}$)$\rangle_{\sigma=0.05}$=%1.3f'%(cos_th_avg_n)+'\n'+r'$\langle$cos($\alpha_{\rm tilt}$)$\rangle_{\sigma=-0.05}$=%1.3f'%(cos_th_avg_p),fontsize=6)
fig.subplots_adjust(left=0.25, bottom=0.21, right=0.96, top=0.95)
fig.savefig('probability_cos_sig%1.2f_kappa%1.4f.pdf'%(sigma_pos,kappa))

# Plot angle
fig = plt.figure(figsize=(3,2), dpi=80)
ax = fig.add_subplot(111)
ax.plot(alpha_plot, prob_p_alpha, c='k', marker='', lw=0.8, ls=':', label=r'$\sigma$=%1.2f C/m$^2$'%sigma_pos)
for axis in ['top','bottom','left','right']:
    ax.spines[axis].set_linewidth(0.5)
ax.set_ylabel('Probability')
ax.set_xlabel(r'$\alpha_{\rm tilt}$')
#ax.set_ylim(top=max(prob_p_alpha)*1.4)
ax.xaxis.set_ticks(arange(0,180+1,45))
ax.legend(loc='upper center', fontsize=7, handlelength=2).get_frame().set_lw(0.3)
#ax.text(75, max(prob_n_cos)/2,r'$\langle \alpha_{\rm tilt}\rangle_{\sigma=0.05}$=%1.3f'%(alpha_avg_n)+'\n'+r'$\langle \alpha_{\rm tilt} \rangle_{\sigma=-0.05}$=%1.3f'%(alpha_avg_p),fontsize=6)
fig.subplots_adjust(left=0.22, bottom=0.21, right=0.93, top=0.95)
fig.savefig('probability_alpha_sig%1.2f_kappa%1.4f.pdf'%(sigma_pos,kappa))

# Scatter plot probability
fig = plt.figure(figsize=(3,2), dpi=80)
ax = fig.add_subplot(111)

fig = plt.figure(figsize=(3,2), dpi=80)
ax = fig.add_subplot(111)

prob_point_p /= Zp_alpha

max_point_prob_p = where(abs(prob_point_p-max(prob_point_p))<1e-10)[0]
print 'Max point probability for sigma=%1.2f is %1.5f at %1.1f tilt, %1.1f rot'%(sigma_pos, prob_point_p[max_point_prob_p], til_point_p[max_point_prob_p], rot_point_p[max_point_prob_p])

til_point_p = reshape(til_point_p,(len(angles_p), len(til_point_p)/len(angles_p)))
rot_point_p = reshape(rot_point_p,(len(angles_p), len(rot_point_p)/len(angles_p)))
prob_point_p = reshape(prob_point_p,(len(angles_p), len(prob_point_p)/len(angles_p)))

rot_point_p_plot = zeros(shape(rot_point_p)+array([0,1]))
rot_point_p_plot[:,0:-1] = rot_point_p[:,:]
rot_point_p_plot[:,-1] = 360 
da_rot = rot_point_p[0,1] - rot_point_p[0,0]
rot_point_p_plot -= da_rot/2

til_point_p_plot = zeros(shape(til_point_p)+array([0,1]))
til_point_p_plot[:,0:-1] = til_point_p[:,:]
til_point_p_plot[:,-1] = til_point_p_plot[:,-2]

#CS=ax.contourf(til_point_p, rot_point_p, prob_point_p)#, s=5, linewidths=0)
CS=ax.pcolor(til_point_p_plot, rot_point_p_plot, prob_point_p, cmap=plt.get_cmap('YlGnBu'))
for axis in ['top','bottom','left','right']:
    ax.spines[axis].set_linewidth(0.5)
ax.xaxis.set_ticks(arange(0,180+1,45))
ax.set_ylabel(r'$\alpha_{\rm rot}$')
ax.set_xlabel(r'$\alpha_{\rm tilt}$')
ax.set_ylim([0,350])
ax.set_xlim([0,180])
ax.set_title(r'Probability for $\sigma$=%1.2f'%sigma_pos)
CB=fig.colorbar(CS)
#print (CB.ax.get_children())
CB.ax.get_children()[2].set_linewidth(0.5)
CB.ax.tick_params(labelsize=6)
fig.subplots_adjust(left=0.17, bottom=0.2, right=0.96, top=0.88)
fig.savefig('probability_point_sig%1.2f_kappa%1.4f.pdf'%(sigma_pos,kappa))
