#!/usr/bin/env python
"""
Code to compute the solvation energy and surface energy of 
protein 1IGT near a charged surface (5 Angs away)

To run:
    from the 'bem_pycuda' directory, run: 

    'PyGBeOrientation_1IGTsampling/run_angle.py param_file config_file tilt_angle rot_angle'

    where:
    0<= tilt_angle <=180
    0<= rot_angle <360

    For sampling results from the paper:
    param_file  = PyGBeOrientation_1IGTsampling/inputs/1IGT.param
    config_file = PyGBeOrientation_1IGTsampling/inputs/1IGT_sensor.config

Output:
    File in 'outputs/' directory with 
    tilt angle, rotation angle, solvation energy, surface energy and sum of energies
"""
from numpy import *
import sys
import os

def scanOutput(filename):
    
    flag = 0 
    files = []
    for line in file(filename):
        line = line.split()
        if len(line)>0:
            if line[0]=='Converged':
                iterations = int(line[2])
            if line[0]=='Total' and line[1]=='elements':
                N = int(line[-1])
            if line[0]=='Totals:':
                flag = 1 
            if line[0]=='Esolv' and flag==1:
                Esolv = float(line[2])
            if line[0]=='Esurf' and flag==1:
                Esurf = float(line[2])
            if line[0]=='Ecoul' and flag==1:
                Ecoul = float(line[2])
            if line[0]=='Time' and flag==1:
                Time = float(line[2])
            if line[0]=='Reading':
                files.append(line[-1])  

    return N, iterations, Esolv, Esurf, Ecoul, Time, files

param_file  = sys.argv[1]
config_file = sys.argv[2]
tilt_angle  = sys.argv[3]
rot_angle   = sys.argv[4]

print '\nParameter file:\n'
f = open(param_file, 'r')
print f.read()
f.close()

name = '_'+tilt_angle+'-'+rot_angle
config_file_moved = config_file[:-7] + name + config_file[-7:]

## Create moved input file
# Works for only 1 pqr file so far (not multiple molecules) and only 1 molecular surface.
fm = open(config_file_moved, 'w')
for line_full in file(config_file):
    line = line_full.split()
    if line[0]=='FILE':
        if line[2]=='dielectric_interface':
            prot_file = line[1]
            prot_file_moved = prot_file + name
            new_line = line[0] + '\t' + prot_file_moved + '\t' + line[2] + '\n'
            fm.write(new_line)
        if line[2]=='neumann_surface' or line[2]=='dirichlet_surface':
            surf_file = line[1]
            phi_file = line[3]
            fm.write(line_full)
    elif line[0]=='FIELD' and int(line[5])>0:
        pqr_file_aux = line[7]
        pqr_file = pqr_file_aux[:-4]
        pqr_file_moved = pqr_file + name + '.pqr'
        new_line = line[0] + '\t'
        for i in range(1,len(line)):
            if i==7:
                new_line += pqr_file_moved + '\t'
            else:
                new_line += line[i] + '\t'
        fm.write(new_line+'\n')

    else:
        fm.write(line_full)

fm.close()

print 'Protein file: ' + prot_file
print 'Sensor  file: ' + surf_file
print 'Phi     file: ' + phi_file + '\n'

N = []
iterations = []
Esolv = []
Esurf = []
Ecoul = []
Time = []


cmd_move = './PyGBeOrientation_1IGTsampling/move_protein.py ' + prot_file + ' ' + pqr_file + ' ' + rot_angle + ' ' + tilt_angle + ' ' + name
os.system(cmd_move)

cmd_run = './main.py ' + param_file + ' ' + config_file_moved +' > output_aux' + name
os.system(cmd_run)

tilt_angle = float(tilt_angle)
rot_angle  = float(rot_angle)

N, iterations, Esolv, Esurf, Ecoul, Time, files = scanOutput('output_aux' + name)

Etotal = Esolv + Esurf + Ecoul
EsurfEsolv = Esolv + Esurf

os.system('rm output_aux' + name + ' ' + config_file_moved + ' ' + prot_file_moved + '.* ' + pqr_file_moved)


output_file = open('PyGBeOrientation_1IGTsampling/outputs/'+name, 'w')

output_file.write('Number of elements  : %i\n'%(N))
output_file.write('Number of iterations: %i\n'%(iterations))
output_file.write('Coulombic energy    : %f kcal/mol\n'%(Ecoul))
output_file.write('Total time          : %fs\n' %(Time))
output_file.write('\n                    ||              kcal/mol\n')
output_file.write('   Tilt   |  Rotat  || Esolv     | Esurf     |    Esurf+Esolv \n')
output_file.write('------------------------------------------------------------------------------\n')
output_file.write('  %3.2f   %3.2f  %s  %s  %s \n'%(tilt_angle, rot_angle, Esolv, Esurf, EsurfEsolv))

output_file.close()
